@@ -240,6 +240,8 @@ struct aws_http_connection_manager {
     struct proxy_env_var_settings proxy_ev_settings;
     struct aws_tls_connection_options *proxy_ev_tls_options;
     uint32_t port;
+    uint64_t response_first_byte_timeout_ms;
+
     /*
      * HTTP/2 specific.
      */
@@ -690,7 +692,8 @@ static void s_aws_http_connection_manager_execute_transaction(struct aws_connect
  * cancels the culling task and then calls this function.  If we are not using idle connection culling, we can
  * call this function immediately from the start of destruction.
  */
-static void s_aws_http_connection_manager_finish_destroy(struct aws_http_connection_manager *manager) {
+static void s_aws_http_connection_manager_finish_destroy(void *user_data) {
+    struct aws_http_connection_manager *manager = user_data;
     if (manager == NULL) {
         return;
     }
@@ -958,6 +961,7 @@ struct aws_http_connection_manager *aws_http_connection_manager_new(
     manager->max_connection_idle_in_milliseconds = options->max_connection_idle_in_milliseconds;
     manager->connection_acquisition_timeout_ms = options->connection_acquisition_timeout_ms;
     manager->max_pending_connection_acquisitions = options->max_pending_connection_acquisitions;
+    manager->response_first_byte_timeout_ms = options->response_first_byte_timeout_ms;
 
     if (options->proxy_ev_settings) {
         manager->proxy_ev_settings = *options->proxy_ev_settings;
@@ -1090,6 +1094,7 @@ static int s_aws_http_connection_manager_new_connection(struct aws_http_connecti
     options.host_name = aws_byte_cursor_from_string(manager->host);
     options.port = manager->port;
     options.initial_window_size = manager->initial_window_size;
+    options.response_first_byte_timeout_ms = manager->response_first_byte_timeout_ms;
     struct aws_socket_options socket_options = manager->socket_options;
     if (aws_array_list_length(&manager->network_interface_names)) {
         struct aws_string *interface_name = NULL;
@@ -1160,7 +1165,6 @@ static void s_aws_http_connection_manager_execute_transaction(struct aws_connect
 
     struct aws_http_connection_manager *manager = work->manager;
 
-    int representative_error = 0;
     size_t new_connection_failures = 0;
 
     /*
@@ -1198,8 +1202,6 @@ static void s_aws_http_connection_manager_execute_transaction(struct aws_connect
      */
     struct aws_array_list errors;
     AWS_ZERO_STRUCT(errors);
-    /* Even if we can't init this array, we still need to invoke error callbacks properly */
-    bool push_errors = false;
 
     if (work->new_connections > 0) {
         AWS_LOGF_INFO(
@@ -1207,20 +1209,21 @@ static void s_aws_http_connection_manager_execute_transaction(struct aws_connect
             "id=%p: Requesting %zu new connections from http",
             (void *)manager,
             work->new_connections);
-        push_errors = aws_array_list_init_dynamic(&errors, work->allocator, work->new_connections, sizeof(int)) ==
-                      AWS_ERROR_SUCCESS;
+        AWS_FATAL_ASSERT(
+            aws_array_list_init_dynamic(&errors, work->allocator, work->new_connections, sizeof(int)) ==
+            AWS_OP_SUCCESS);
     }
 
     for (size_t i = 0; i < work->new_connections; ++i) {
         if (s_aws_http_connection_manager_new_connection(manager)) {
             ++new_connection_failures;
-            representative_error = aws_last_error();
-            if (push_errors) {
-                AWS_FATAL_ASSERT(aws_array_list_push_back(&errors, &representative_error) == AWS_OP_SUCCESS);
-            }
+            int error = aws_last_error();
+            AWS_FATAL_ASSERT(aws_array_list_push_back(&errors, &error) == AWS_OP_SUCCESS);
         }
     }
 
+    bool has_pending_acquisitions = false;
+    struct aws_connection_management_transaction pending_acquisitions_work;
     if (new_connection_failures > 0) {
         /*
          * We failed and aren't going to receive a callback, but the current state assumes we will receive
@@ -1231,30 +1234,24 @@ static void s_aws_http_connection_manager_execute_transaction(struct aws_connect
         AWS_FATAL_ASSERT(manager->internal_ref[AWS_HCMCT_PENDING_CONNECTIONS] >= new_connection_failures);
         s_connection_manager_internal_ref_decrease(manager, AWS_HCMCT_PENDING_CONNECTIONS, new_connection_failures);
 
-        /*
-         * Rather than failing one acquisition for each connection failure, if there's at least one
-         * connection failure, we instead fail all excess acquisitions, since there's no pending
-         * connect that will necessarily resolve them.
-         *
-         * Try to correspond an error with the acquisition failure, but as a fallback just use the
-         * representative error.
-         */
-        size_t i = 0;
-        while (manager->pending_acquisition_count > manager->internal_ref[AWS_HCMCT_PENDING_CONNECTIONS]) {
-            int error = representative_error;
-            if (i < aws_array_list_length(&errors)) {
-                aws_array_list_get_at(&errors, &error, i);
-            }
-
+        for (size_t i = 0; i < new_connection_failures && manager->pending_acquisition_count > 0; i++) {
+            int error;
+            aws_array_list_get_at(&errors, &error, i);
             AWS_LOGF_DEBUG(
                 AWS_LS_HTTP_CONNECTION_MANAGER,
-                "id=%p: Failing excess connection acquisition with error code %d",
+                "id=%p: Failing connection acquisition with error code %d",
                 (void *)manager,
                 (int)error);
             s_aws_http_connection_manager_move_front_acquisition(manager, NULL, error, &work->completions);
-            ++i;
         }
-
+        has_pending_acquisitions =
+            manager->internal_ref[AWS_HCMCT_PENDING_CONNECTIONS] + manager->pending_settings_count <
+            manager->pending_acquisition_count;
+        if (has_pending_acquisitions) {
+            /* If there are pending acquisitions, schedule work again to try acquiring them */
+            s_aws_connection_management_transaction_init(&pending_acquisitions_work, manager);
+            s_aws_http_connection_manager_build_transaction(&pending_acquisitions_work);
+        }
         aws_mutex_unlock(&manager->lock);
     }
 
@@ -1262,13 +1259,20 @@ static void s_aws_http_connection_manager_execute_transaction(struct aws_connect
      * Step 4 - Perform acquisition callbacks
      */
     s_aws_http_connection_manager_complete_acquisitions(&work->completions, work->allocator);
-
     aws_array_list_clean_up(&errors);
 
     /*
      * Step 5 - Clean up work.  Do this here rather than at the end of every caller. Destroy the manager if necessary
      */
     s_aws_connection_management_transaction_clean_up(work);
+
+    /*
+     * Step 6 - If some connection acquisition requests failed and we still have more pending acquisitions, try to
+     * acquire them.
+     */
+    if (has_pending_acquisitions) {
+        s_aws_http_connection_manager_execute_transaction(&pending_acquisitions_work);
+    }
 }
 
 void aws_http_connection_manager_acquire_connection(
@@ -1311,7 +1315,8 @@ void aws_http_connection_manager_acquire_connection(
     AWS_FATAL_ASSERT(manager->state == AWS_HCMST_READY);
 
     if (manager->max_pending_connection_acquisitions == 0 ||
-        manager->pending_acquisition_count < manager->max_pending_connection_acquisitions) {
+        manager->pending_acquisition_count + manager->internal_ref[AWS_HCMCT_VENDED_CONNECTION] <
+            manager->max_pending_connection_acquisitions + manager->max_connections) {
         aws_linked_list_push_back(&manager->pending_acquisitions, &request->node);
         ++manager->pending_acquisition_count;
     } else {
@@ -1474,12 +1479,11 @@ static void s_cm_on_connection_ready_or_failed(
             work->connection_to_release = connection;
         }
     } else {
-        /* fail acquisition as one connection cannot be used any more */
-        while (manager->pending_acquisition_count >
-               manager->internal_ref[AWS_HCMCT_PENDING_CONNECTIONS] + manager->pending_settings_count) {
+        if (manager->pending_acquisition_count > 0) {
+            /* fail acquisition as connection acquire failed */
             AWS_LOGF_DEBUG(
                 AWS_LS_HTTP_CONNECTION_MANAGER,
-                "id=%p: Failing excess connection acquisition with error code %d",
+                "id=%p: Failing connection acquisition with error code %d",
                 (void *)manager,
                 (int)error_code);
             s_aws_http_connection_manager_move_front_acquisition(manager, NULL, error_code, &work->completions);
