@@ -10,14 +10,12 @@
 
 #include <aws/common/clock.h>
 #include <aws/common/condition_variable.h>
-#include <aws/common/log_writer.h>
+#include <aws/common/hash_table.h>
 #include <aws/common/logging.h>
 #include <aws/common/string.h>
 #include <aws/common/thread.h>
-#include <aws/common/uuid.h>
 #include <aws/io/channel_bootstrap.h>
 #include <aws/io/event_loop.h>
-#include <aws/io/logging.h>
 #include <aws/io/socket.h>
 #include <aws/io/tls_channel_handler.h>
 #include <aws/testing/aws_test_harness.h>
@@ -737,14 +735,11 @@ static int s_test_connection_customized_alpn_error_with_unknown_return_string(
     tester.wait_client_connection_num = 1;
     tester.wait_server_connection_num = 1;
 
-#ifndef __APPLE__ /* Server side ALPN doesn't work for MacOS */
     ASSERT_FAILS(s_tester_wait(&tester, s_tester_connection_setup_pred));
     /* Assert that we have the negotiated protocol and error returned from callback */
     ASSERT_TRUE(aws_byte_buf_eq_c_str(&tester.negotiated_protocol, customized_alpn_string));
     ASSERT_INT_EQUALS(aws_last_error(), AWS_ERROR_HTTP_UNSUPPORTED_PROTOCOL);
-#else
-    ASSERT_SUCCESS(s_tester_wait(&tester, s_tester_connection_setup_pred));
-#endif
+
     /* clean up */
     release_all_client_connections(&tester);
     release_all_server_connections(&tester);
